@@ -3,7 +3,7 @@
 
 #include "altrep.h"
 
-#if (R_VERSION >= R_Version(3, 5, 0))
+#if (HAS_ALTREP)
 
 SEXP altrep_rle_Make(SEXP input);
 R_xlen_t altrep_rle_Length(SEXP vec);
@@ -20,7 +20,7 @@ void* altrep_rle_Dataptr(SEXP vec, Rboolean writeable);
 const void* altrep_rle_Dataptr_or_null(SEXP vec);
 void vctrs_init_altrep_rle(DllInfo* dll);
 
-R_altrep_class_t altrep_rle_class;
+extern R_altrep_class_t altrep_rle_class;
 
 #endif
 
