@@ -1,9 +1,8 @@
 #include <ctype.h>
-
 #include "vctrs.h"
+#include "type-data-frame.h"
 #include "utils.h"
-
-#include <ctype.h>
+#include "dim.h"
 
 static void describe_repair(SEXP old_names, SEXP new_names);
 
@@ -19,7 +18,7 @@ SEXP fns_validate_unique_names = NULL;
 // Defined below
 SEXP vctrs_as_minimal_names(SEXP names);
 SEXP vec_as_universal_names(SEXP names, bool quiet);
-SEXP vec_validate_unique_names(SEXP names);
+SEXP vec_validate_unique_names(SEXP names, struct vctrs_arg* arg);
 SEXP vec_as_custom_names(SEXP names, const struct name_repair_opts* opts);
 static void vec_validate_minimal_names(SEXP names, R_len_t n);
 
@@ -34,19 +33,22 @@ SEXP vec_as_names(SEXP names, const struct name_repair_opts* opts) {
   case name_repair_minimal: return vctrs_as_minimal_names(names);
   case name_repair_unique: return vec_as_unique_names(names, opts->quiet);
   case name_repair_universal: return vec_as_universal_names(names, opts->quiet);
-  case name_repair_check_unique: return vec_validate_unique_names(names);
+  case name_repair_check_unique: return vec_validate_unique_names(names, opts->arg);
   case name_repair_custom: return vec_as_custom_names(names, opts);
   }
   never_reached("vec_as_names");
 }
 
 // [[ register() ]]
-SEXP vctrs_as_names(SEXP names, SEXP repair, SEXP quiet) {
+SEXP vctrs_as_names(SEXP names, SEXP repair, SEXP repair_arg, SEXP quiet) {
   if (!r_is_bool(quiet)) {
     Rf_errorcall(R_NilValue, "`quiet` must a boolean value.");
   }
+  bool quiet_ = LOGICAL(quiet)[0];
 
-  struct name_repair_opts repair_opts = new_name_repair_opts(repair, quiet);
+  struct vctrs_arg arg_ = vec_as_arg(repair_arg);
+
+  struct name_repair_opts repair_opts = new_name_repair_opts(repair, &arg_, quiet_);
   PROTECT_NAME_REPAIR_OPTS(&repair_opts);
 
   SEXP out = vec_as_names(names, &repair_opts);
@@ -63,14 +65,17 @@ SEXP vec_as_universal_names(SEXP names, bool quiet) {
   UNPROTECT(1);
   return out;
 }
-SEXP vec_validate_unique_names(SEXP names) {
-  SEXP out = PROTECT(vctrs_dispatch1(syms_validate_unique_names, fns_validate_unique_names,
-                                     syms_names, names));
+SEXP vec_validate_unique_names(SEXP names, struct vctrs_arg* arg) {
+  SEXP arg_obj = PROTECT(vctrs_arg(arg));
+
+  SEXP out = PROTECT(vctrs_dispatch2(syms_validate_unique_names, fns_validate_unique_names,
+                                     syms_names, names,
+                                     syms_arg, arg_obj));
 
   // Restore visibility
   Rf_eval(R_NilValue, R_EmptyEnv);
 
-  UNPROTECT(1);
+  UNPROTECT(2);
   return out;
 }
 
@@ -79,37 +84,41 @@ SEXP vec_as_custom_names(SEXP names, const struct name_repair_opts* opts) {
 
   // Don't use vctrs dispatch utils because we match argument positionally
   SEXP call = PROTECT(Rf_lang2(syms_repair, syms_names));
-  SEXP mask = PROTECT(r_new_environment(R_GlobalEnv, 2));
+  SEXP mask = PROTECT(r_new_environment(R_GlobalEnv));
   Rf_defineVar(syms_repair, opts->fn, mask);
   Rf_defineVar(syms_names, names, mask);
   SEXP out = PROTECT(Rf_eval(call, mask));
 
   vec_validate_minimal_names(out, Rf_length(names));
 
-  if (!opts->quiet) {
-    describe_repair(names, out);
-  }
-
   UNPROTECT(4);
   return out;
 }
 
+static
+SEXP vec_names_impl(SEXP x, bool proxy) {
+  bool has_class = OBJECT(x);
 
-// [[ register(); include("vctrs.h") ]]
-SEXP vec_names(SEXP x) {
-  if (OBJECT(x) && Rf_inherits(x, "data.frame")) {
-    return R_NilValue;
+  if (has_class && Rf_inherits(x, "data.frame")) {
+    // Only return row names if they are character. Data frames with
+    // automatic row names are treated as unnamed.
+    SEXP rn = df_rownames(x);
+    if (rownames_type(rn) == ROWNAMES_IDENTIFIERS) {
+      return rn;
+    } else {
+      return R_NilValue;
+    }
   }
 
-  if (vec_dim_n(x) == 1) {
-    if (OBJECT(x)) {
+  if (vec_bare_dim(x) == R_NilValue) {
+    if (!proxy && has_class) {
       return vctrs_dispatch1(syms_names, fns_names, syms_x, x);
     } else {
       return r_names(x);
     }
   }
 
-  SEXP dimnames = PROTECT(Rf_getAttrib(x, R_DimNamesSymbol));
+  SEXP dimnames = PROTECT(r_attrib_get(x, R_DimNamesSymbol));
   if (dimnames == R_NilValue || Rf_length(dimnames) < 1) {
     UNPROTECT(1);
     return R_NilValue;
@@ -120,6 +129,15 @@ SEXP vec_names(SEXP x) {
   return out;
 }
 
+// [[ register(); include("vctrs.h") ]]
+SEXP vec_names(SEXP x) {
+  return vec_names_impl(x, false);
+}
+// [[ include("vctrs.h") ]]
+SEXP vec_proxy_names(SEXP x) {
+  return vec_names_impl(x, true);
+}
+
 // [[ register() ]]
 SEXP vctrs_as_minimal_names(SEXP names) {
   if (TYPEOF(names) != STRSXP) {
@@ -371,7 +389,7 @@ static ptrdiff_t suffix_pos(const char* name) {
       goto done;
 
     default:
-      Rf_error("Internal error: Unexpected state in `suffix_pos()`");
+      stop_internal("suffix_pos", "Unexpected state.");
     }}
 
  done:
@@ -461,10 +479,10 @@ static void describe_repair(SEXP old_names, SEXP new_names) {
 // [[ register() ]]
 SEXP vctrs_outer_names(SEXP names, SEXP outer, SEXP n) {
   if (names != R_NilValue && TYPEOF(names) != STRSXP) {
-    Rf_error("Internal error: `names` must be `NULL` or a string");
+    stop_internal("vctrs_outer_names", "`names` must be `NULL` or a string.");
   }
   if (!r_is_number(n)) {
-    Rf_error("Internal error: `n` must be a single integer");
+    stop_internal("vctrs_outer_names", "`n` must be a single integer.");
   }
 
   if (outer != R_NilValue) {
@@ -480,7 +498,7 @@ SEXP outer_names(SEXP names, SEXP outer, R_len_t n) {
     return names;
   }
   if (TYPEOF(outer) != CHARSXP) {
-    Rf_error("Internal error: `outer` must be a scalar string.");
+    stop_internal("outer_names", "`outer` must be a scalar string.");
   }
 
   if (outer == strings_empty || outer == NA_STRING) {
@@ -507,18 +525,29 @@ static SEXP glue_as_name_spec(SEXP spec);
 
 // [[ include("utils.h") ]]
 SEXP apply_name_spec(SEXP name_spec, SEXP outer, SEXP inner, R_len_t n) {
+  if (Rf_inherits(name_spec, "rlang_zap")) {
+    return R_NilValue;
+  }
+
   if (outer == R_NilValue) {
     return inner;
   }
   if (TYPEOF(outer) != CHARSXP) {
-    Rf_error("Internal error: `outer` must be a scalar string.");
+    stop_internal("apply_name_spec", "`outer` must be a scalar string.");
   }
 
   if (outer == strings_empty || outer == NA_STRING) {
-    return inner;
+    if (inner == R_NilValue) {
+      return chrs_empty;
+    } else {
+      return inner;
+    }
   }
 
   if (r_is_empty_names(inner)) {
+    if (n == 0) {
+      return vctrs_shared_empty_chr;
+    }
     if (n == 1) {
       return r_str_as_character(outer);
     }
@@ -544,22 +573,23 @@ SEXP apply_name_spec(SEXP name_spec, SEXP outer, SEXP inner, R_len_t n) {
   }
   PROTECT(name_spec);
 
-  // Recycle `outer` so specs don't need to refer to both `outer` and `inner`
-  SEXP outer_chr = PROTECT(Rf_allocVector(STRSXP, n));
-  r_chr_fill(outer_chr, outer, n);
+  SEXP outer_chr = PROTECT(r_str_as_character(outer));
 
-  SEXP out = vctrs_dispatch2(syms_dot_name_spec, name_spec,
-                             syms_outer, outer_chr,
-                             syms_inner, inner);
+  SEXP out = PROTECT(vctrs_dispatch2(syms_dot_name_spec, name_spec,
+                                     syms_outer, outer_chr,
+                                     syms_inner, inner));
+  out = vec_recycle(out, n, NULL);
 
-  if (TYPEOF(out) != STRSXP) {
-    Rf_errorcall(R_NilValue, "`.name_spec` must return a character vector.");
-  }
-  if (Rf_length(out) != n) {
-    Rf_errorcall(R_NilValue, "`.name_spec` must return a character vector as long as `inner`.");
+  if (out != R_NilValue) {
+    if (TYPEOF(out) != STRSXP) {
+      Rf_errorcall(R_NilValue, "`.name_spec` must return a character vector.");
+    }
+    if (Rf_length(out) != n) {
+      Rf_errorcall(R_NilValue, "`.name_spec` must return a character vector as long as `inner`.");
+    }
   }
 
-  UNPROTECT(3);
+  UNPROTECT(4);
   return out;
 }
 
@@ -576,10 +606,14 @@ static SEXP glue_as_name_spec(SEXP spec) {
                          syms_internal_spec, spec);
 }
 
+#define VCTRS_PASTE_BUFFER_MAX_SIZE 4096
+char vctrs_paste_buffer[VCTRS_PASTE_BUFFER_MAX_SIZE];
 
 // [[ include("names.h") ]]
 SEXP r_chr_paste_prefix(SEXP names, const char* prefix, const char* sep) {
-  names = PROTECT(Rf_shallow_duplicate(names));
+  int n_protect = 0;
+
+  names = PROTECT_N(Rf_shallow_duplicate(names), &n_protect);
   R_len_t n = Rf_length(names);
 
   int outer_len = strlen(prefix);
@@ -588,8 +622,15 @@ SEXP r_chr_paste_prefix(SEXP names, const char* prefix, const char* sep) {
   int sep_len = strlen(sep);
   int total_len = outer_len + names_len + sep_len + 1;
 
-  R_CheckStack2(total_len);
-  char buf[total_len];
+  char* buf = vctrs_paste_buffer;
+  if (total_len > VCTRS_PASTE_BUFFER_MAX_SIZE) {
+    SEXP buf_box = PROTECT_N(
+      Rf_allocVector(RAWSXP, total_len * sizeof(char)),
+      &n_protect
+    );
+    buf = (char*) RAW(buf_box);
+  }
+
   buf[total_len - 1] = '\0';
   char* bufp = buf;
 
@@ -599,10 +640,10 @@ SEXP r_chr_paste_prefix(SEXP names, const char* prefix, const char* sep) {
     *bufp++ = sep[i];
   }
 
-  SEXP* p = STRING_PTR(names);
+  SEXP const* p_names = STRING_PTR_RO(names);
 
-  for (R_len_t i = 0; i < n; ++i, ++p) {
-    const char* inner = CHAR(*p);
+  for (R_len_t i = 0; i < n; ++i) {
+    const char* inner = CHAR(p_names[i]);
     int inner_n = strlen(inner);
 
     memcpy(bufp, inner, inner_n);
@@ -611,10 +652,17 @@ SEXP r_chr_paste_prefix(SEXP names, const char* prefix, const char* sep) {
     SET_STRING_ELT(names, i, r_str(buf));
   }
 
-  UNPROTECT(1);
+  UNPROTECT(n_protect);
   return names;
 }
 
+// [[ register() ]]
+SEXP vctrs_chr_paste_prefix(SEXP names, SEXP prefix, SEXP sep) {
+  return r_chr_paste_prefix(names,
+                            r_chr_get_c_string(prefix, 0),
+                            r_chr_get_c_string(sep, 0));
+}
+
 // [[ include("names.h") ]]
 SEXP r_seq_chr(const char* prefix, R_len_t n) {
   int total_len = 24 + strlen(prefix) + 1;
@@ -672,8 +720,8 @@ static void check_names(SEXP x, SEXP names) {
   }
 }
 
-SEXP vec_set_rownames(SEXP x, SEXP names) {
-  if (OBJECT(x)) {
+SEXP vec_set_rownames(SEXP x, SEXP names, bool proxy, const enum vctrs_owned owned) {
+  if (!proxy && OBJECT(x)) {
     return set_rownames_fallback(x, names);
   }
 
@@ -688,10 +736,13 @@ SEXP vec_set_rownames(SEXP x, SEXP names) {
     }
   }
 
-  x = PROTECT_N(r_maybe_duplicate(x), &nprot);
+  x = PROTECT_N(vec_clone_referenced(x, owned), &nprot);
 
   if (dim_names == R_NilValue) {
     dim_names = PROTECT_N(Rf_allocVector(VECSXP, vec_dim_n(x)), &nprot);
+  } else {
+    // Also clone attribute
+    dim_names = PROTECT_N(Rf_shallow_duplicate(dim_names), &nprot);
   }
 
   SET_VECTOR_ELT(dim_names, 0, names);
@@ -702,22 +753,46 @@ SEXP vec_set_rownames(SEXP x, SEXP names) {
   return x;
 }
 
-// FIXME: Do we need to get the vec_proxy() and only fall back if it doesn't
-// exist? See #526 and #531 for discussion and the related issue.
-// [[ include("utils.h"); register() ]]
-SEXP vec_set_names(SEXP x, SEXP names) {
-  // Never on a data frame
-  if (is_data_frame(x)) {
+SEXP vec_set_df_rownames(SEXP x, SEXP names, bool proxy, const enum vctrs_owned owned) {
+  if (names == R_NilValue) {
+    if (rownames_type(df_rownames(x)) != ROWNAMES_IDENTIFIERS) {
+      return(x);
+    }
+
+    x = PROTECT(vec_clone_referenced(x, owned));
+    init_compact_rownames(x, vec_size(x));
+
+    UNPROTECT(1);
     return x;
   }
 
+  // Repair row names silently
+  if (!proxy) {
+    names = vec_as_names(names, p_unique_repair_silent_opts);
+  }
+  PROTECT(names);
+
+  x = PROTECT(vec_clone_referenced(x, owned));
+  Rf_setAttrib(x, R_RowNamesSymbol, names);
+
+  UNPROTECT(2);
+  return x;
+}
+
+// FIXME: Do we need to get the vec_proxy() and only fall back if it doesn't
+// exist? See #526 and #531 for discussion and the related issue.
+SEXP vec_set_names_impl(SEXP x, SEXP names, bool proxy, const enum vctrs_owned owned) {
   check_names(x, names);
 
-  if (has_dim(x)) {
-    return vec_set_rownames(x, names);
+  if (is_data_frame(x)) {
+    return vec_set_df_rownames(x, names, proxy, owned);
   }
 
-  if (OBJECT(x)) {
+  if (has_dim(x)) {
+    return vec_set_rownames(x, names, proxy, owned);
+  }
+
+  if (!proxy && OBJECT(x)) {
     return set_names_fallback(x, names);
   }
 
@@ -726,16 +801,24 @@ SEXP vec_set_names(SEXP x, SEXP names) {
     return x;
   }
 
-  x = PROTECT(r_maybe_duplicate(x));
-
+  x = PROTECT(vec_clone_referenced(x, owned));
   Rf_setAttrib(x, R_NamesSymbol, names);
 
   UNPROTECT(1);
   return x;
 }
+// [[ include("utils.h"); register() ]]
+SEXP vec_set_names(SEXP x, SEXP names) {
+  return vec_set_names_impl(x, names, false, VCTRS_OWNED_false);
+}
+// [[ include("utils.h") ]]
+SEXP vec_proxy_set_names(SEXP x, SEXP names, const enum vctrs_owned owned) {
+  return vec_set_names_impl(x, names, true, owned);
+}
+
 
 SEXP vctrs_validate_name_repair_arg(SEXP arg) {
-  struct name_repair_opts opts = new_name_repair_opts(arg, true);
+  struct name_repair_opts opts = new_name_repair_opts(arg, args_empty, true);
   if (opts.type == name_repair_custom) {
     return opts.fn;
   } else if (Rf_length(arg) != 1) {
@@ -749,10 +832,11 @@ void stop_name_repair() {
   Rf_errorcall(R_NilValue, "`.name_repair` must be a string or a function. See `?vctrs::vec_as_names`.");
 }
 
-struct name_repair_opts new_name_repair_opts(SEXP name_repair, bool quiet) {
+struct name_repair_opts new_name_repair_opts(SEXP name_repair, struct vctrs_arg* arg, bool quiet) {
   struct name_repair_opts opts = {
     .type = 0,
     .fn = R_NilValue,
+    .arg = arg,
     .quiet = quiet
   };
 
@@ -775,7 +859,7 @@ struct name_repair_opts new_name_repair_opts(SEXP name_repair, bool quiet) {
     } else if (c == strings_check_unique) {
       opts.type = name_repair_check_unique;
     } else {
-      Rf_errorcall(R_NilValue, "`.name_repair` can't be \"%s\". See `?vctrs::vec_as_names`.", CHAR(name_repair));
+      Rf_errorcall(R_NilValue, "`.name_repair` can't be \"%s\". See `?vctrs::vec_as_names`.", CHAR(c));
     }
 
     return opts;
@@ -836,7 +920,7 @@ SEXP vctrs_validate_minimal_names(SEXP names, SEXP n_) {
 
   if (TYPEOF(n_) == INTSXP) {
     if (Rf_length(n_) != 1) {
-      Rf_error("Internal error (minimal names validation): `n` must be a single number.");
+      stop_internal("vctrs_validate_minimal_names", "`n` must be a single number.");
     }
     n = INTEGER(n_)[0];
   }
@@ -846,7 +930,8 @@ SEXP vctrs_validate_minimal_names(SEXP names, SEXP n_) {
 }
 
 
-struct name_repair_opts default_unique_repair_opts_obj;
+struct name_repair_opts unique_repair_default_opts;
+struct name_repair_opts unique_repair_silent_opts;
 
 void vctrs_init_names(SEXP ns) {
   syms_set_rownames_fallback = Rf_install("set_rownames_fallback");
@@ -863,7 +948,11 @@ void vctrs_init_names(SEXP ns) {
   fns_glue_as_name_spec = r_env_get(ns, syms_glue_as_name_spec);
   syms_internal_spec = Rf_install("_spec");
 
-  default_unique_repair_opts_obj.type = name_repair_unique;
-  default_unique_repair_opts_obj.fn = R_NilValue;
-  default_unique_repair_opts_obj.quiet = false;
+  unique_repair_default_opts.type = name_repair_unique;
+  unique_repair_default_opts.fn = R_NilValue;
+  unique_repair_default_opts.quiet = false;
+
+  unique_repair_silent_opts.type = name_repair_unique;
+  unique_repair_silent_opts.fn = R_NilValue;
+  unique_repair_silent_opts.quiet = true;
 }
