@@ -4,8 +4,10 @@ test_that("vec_slice throws error with non-vector inputs", {
 })
 
 test_that("vec_slice throws error with non-vector subscripts", {
-  expect_error(vec_slice(1:3, Sys.Date()), class = "vctrs_error_incompatible_cast")
-  expect_error(vec_slice(1:3, matrix(TRUE, nrow = 1)), "must have one dimension")
+  verify_errors({
+    expect_error(vec_slice(1:3, Sys.Date()), class = "vctrs_error_subscript_type")
+    expect_error(vec_slice(1:3, matrix(TRUE, nrow = 1)), class = "vctrs_error_subscript_type")
+  })
 })
 
 test_that("can subset base vectors", {
@@ -180,20 +182,13 @@ test_that("0 is ignored in positive indices", {
 test_that("can slice with double indices", {
   expect_identical(vec_slice(1:3, dbl(2, 3)), 2:3)
   err <- expect_error(vec_as_location(2^31, 3L), class = "vctrs_error_subscript_type")
-  expect_is(err$parent, "vctrs_error_cast_lossy")
+  expect_s3_class(err$parent, "vctrs_error_cast_lossy")
 })
 
 test_that("can slice with symbols", {
   expect_identical(vec_as_location(quote(b), 26, letters), 2L)
 })
 
-test_that("vec_as_location() checks type", {
-  expect_error(vec_as_location("foo", "bar"), class = "vctrs_error_incompatible_type")
-  expect_error(vec_as_location("foo", 1L, names = 1L), "must be a character vector")
-  expect_error(vec_as_location(Sys.Date(), 3L), class = "vctrs_error_subscript_type")
-  expect_error(vec_as_location(matrix(TRUE, nrow = 1), 3L), "must have one dimension")
-})
-
 test_that("can `vec_slice()` S3 objects without dispatch infloop", {
   expect_identical(new_vctr(1:3)[1], new_vctr(1L))
   expect_identical(new_vctr(as.list(1:3))[1], new_vctr(list(1L)))
@@ -222,16 +217,28 @@ test_that("vec_slice() is proxied", {
 })
 
 test_that("dimensions are preserved by vec_slice()", {
-  attrib <- NULL
-
-  local_methods(
-    vec_restore.vctrs_foobar = function(x, ...) attrib <<- attributes(x)
-  )
-
+  # Fallback case
   x <- foobar(1:4)
   dim(x) <- c(2, 2)
   dimnames(x) <- list(a = c("foo", "bar"), b = c("quux", "hunoz"))
 
+  out <- vec_slice(x, 1)
+  exp <- foobar(
+    c(1L, 3L),
+    dim = c(1, 2),
+    dimnames = list(a = "foo", b = c("quux", "hunoz")
+  ))
+  expect_identical(out, exp)
+
+
+  # Native case
+  attrib <- NULL
+
+  local_methods(
+    vec_proxy.vctrs_foobar = identity,
+    vec_restore.vctrs_foobar = function(x, to, ...) attrib <<- attributes(x)
+  )
+
   vec_slice(x, 1)
 
   exp <- list(dim = 1:2, dimnames = list(a = "foo", b = c("quux", "hunoz")))
@@ -276,11 +283,21 @@ test_that("can call `vec_slice()` from `[` methods with shaped objects without i
   expect_identical(x[1], exp)
 })
 
+test_that("vec_slice() restores attributes on shaped S3 objects correctly", {
+  x <- factor(c("a", "b", "c", "d", "e", "f"))
+  dim(x) <- c(3, 2)
+
+  expect <- factor(c("a", "c", "d", "f"), levels = levels(x))
+  dim(expect) <- c(2, 2)
+
+  expect_identical(vec_slice(x, c(1, 3)), expect)
+})
+
 test_that("vec_slice() falls back to `[` with S3 objects", {
   local_methods(
     `[.vctrs_foobar` = function(x, i, ...) "dispatched"
   )
-  expect_identical(vec_slice(foobar(NA), 1), foobar("dispatched"))
+  expect_identical(vec_slice(foobar(NA), 1), "dispatched")
 
   expect_error(vec_slice(foobar(list(NA)), 1), class = "vctrs_error_scalar_type")
   local_methods(
@@ -297,6 +314,14 @@ test_that("vec_slice() doesn't restore when attributes have already been restore
   expect_error(vec_slice(foobar(NA), 1), NA)
 })
 
+test_that("vec_slice() doesn't restore when `[` method intentionally dropped attributes", {
+  local_methods(
+    `[.vctrs_foobar` = function(x, i, ...) unstructure(NextMethod()),
+    vec_restore.vctrs_foobar = function(...) stop("not called")
+  )
+  expect_identical(vec_slice(foobar(NA), 1), NA)
+})
+
 test_that("can vec_slice() without inflooping when restore calls math generics", {
   local_methods(
     new_foobar = function(x) {
@@ -383,10 +408,6 @@ test_that("missing indices don't create NA names", {
   expect_identical(vec_slice(x, 1:2), x)
 })
 
-test_that("vec_slice throws error with non-vector inputs", {
-  expect_error(vec_slice(environment(), 1L), class = "vctrs_error_scalar_type")
-})
-
 test_that("vec_slice() asserts vectorness (#301)", {
   expect_error(vec_slice(NULL, 1), class = "vctrs_error_scalar_type")
 })
@@ -401,6 +422,44 @@ test_that("slicing an unspecified() object returns an unspecified()", {
   expect_identical(vec_slice(unspecified(1), c(1, 1)), unspecified(2))
 })
 
+
+test_that("vec_slice() works with Altrep classes with custom extract methods", {
+  skip_if(getRversion() < "3.5")
+
+  x <- .Call(vctrs_altrep_rle_Make, c(foo = 10L, bar = 5L))
+
+  idx <- c(9, 10, 11)
+  expect_equal(vec_slice(x, idx), c("foo", "foo", "bar"))
+})
+
+test_that("slice has informative error messages", {
+  verify_output(test_path("error", "test-slice.txt"), {
+    "# Unnamed vector with character subscript"
+    vec_slice(1:3, letters[1])
+
+    "# Negative subscripts are checked"
+    vec_slice(1:3, -c(1L, NA))
+    vec_slice(1:3, c(-1L, 1L))
+
+    "# oob error messages are properly constructed"
+    vec_slice(c(bar = 1), "foo")
+
+    "Multiple OOB indices"
+    vec_slice(letters, c(100, 1000))
+    vec_slice(letters, c(1, 100:103, 2, 104:110))
+    vec_slice(set_names(letters), c("foo", "bar"))
+    vec_slice(set_names(letters), toupper(letters))
+
+    "# Can't index beyond the end of a vector"
+    vec_slice(1:2, 3L)
+    vec_slice(1:2, -3L)
+
+    "# vec_slice throws error with non-vector subscripts"
+    vec_slice(1:3, Sys.Date())
+    vec_slice(1:3, matrix(TRUE, ncol = 1))
+  })
+})
+
 # vec_init ----------------------------------------------------------------
 
 test_that("na of atomic vectors is as expected", {
@@ -449,194 +508,11 @@ test_that("vec_init() asserts vectorness (#301)", {
 test_that("vec_init() works with Altrep classes", {
   skip_if(getRversion() < "3.5")
 
-  x <- .Call(vctrs_rle, c(foo = 1L, bar = 2L))
+  x <- .Call(vctrs_altrep_rle_Make, c(foo = 1L, bar = 2L))
 
   expect_equal(vec_init(x, 2), rep(NA_character_, 2))
 })
 
-# vec_chop ----------------------------------------------------------------
-
-test_that("vec_chop() throws error with non-vector inputs", {
-  expect_error(vec_chop(NULL), class = "vctrs_error_scalar_type")
-  expect_error(vec_chop(environment()), class = "vctrs_error_scalar_type")
-})
-
-test_that("atomics are split into a list", {
-  x <- 1:5
-  expect_equal(vec_chop(x), as.list(x))
-
-  x <- letters[1:5]
-  expect_equal(vec_chop(x), as.list(x))
-})
-
-test_that("atomic names are kept", {
-  x <- set_names(1:5)
-  result <- lapply(vec_chop(x), names)
-  expect_equal(result, as.list(names(x)))
-})
-
-test_that("base R classed objects are split into a list", {
-  fctr <- factor(c("a", "b"))
-  expect <- lapply(vec_seq_along(fctr), vec_slice, x = fctr)
-  expect_equal(vec_chop(fctr), expect)
-
-  date <- new_date(c(0, 1))
-  expect <- lapply(vec_seq_along(date), vec_slice, x = date)
-  expect_equal(vec_chop(date), expect)
-})
-
-test_that("base R classed object names are kept", {
-  fctr <- set_names(factor(c("a", "b")))
-  result <- lapply(vec_chop(fctr), names)
-  expect_equal(result, as.list(names(fctr)))
-})
-
-test_that("list elements are split", {
-  x <- list(1, 2)
-  result <- list(vec_slice(x, 1), vec_slice(x, 2))
-  expect_equal(vec_chop(x), result)
-})
-
-test_that("data frames are split rowwise", {
-  x <- data_frame(x = 1:2, y = c("a", "b"))
-  result <- list(vec_slice(x, 1), vec_slice(x, 2))
-  expect_equal(vec_chop(x), result)
-})
-
-test_that("data frame row names are kept", {
-  x <- data_frame(x = 1:2, y = c("a", "b"))
-  rownames(x) <- c("r1", "r2")
-  result <- lapply(vec_chop(x), rownames)
-  expect_equal(result, list("r1", "r2"))
-})
-
-test_that("matrices / arrays are split rowwise", {
-  x <- array(1:12, c(2, 2, 2))
-  result <- list(vec_slice(x, 1), vec_slice(x, 2))
-  expect_equal(vec_chop(x), result)
-})
-
-test_that("matrix / array row names are kept", {
-  x <- array(1:12, c(2, 2, 2), dimnames = list(c("r1", "r2"), c("c1", "c2")))
-  result <- lapply(vec_chop(x), rownames)
-  expect_equal(result, list("r1", "r2"))
-})
-
-test_that("matrices / arrays without row names have other dimension names kept", {
-  x <- array(1:12, c(2, 2, 2), dimnames = list(NULL, c("c1", "c2")))
-  result <- lapply(vec_chop(x), colnames)
-  expect_equal(result, list(c("c1", "c2"), c("c1", "c2")))
-})
-
-test_that("vec_chop() doesn't restore when attributes have already been restored", {
-  local_methods(
-    `[.vctrs_foobar` = function(x, i, ...) structure("dispatched", foo = "bar"),
-    vec_restore.vctrs_foobar = function(...) structure("dispatched-and-restored", foo = "bar")
-  )
-
-  result <- vec_chop(foobar(NA))[[1]]
-  expect_equal(result, structure("dispatched", foo = "bar"))
-})
-
-test_that("vec_chop() restores when attributes have not been restored by `[`", {
-  local_methods(
-    `[.vctrs_foobar` = function(x, i, ...) "dispatched",
-    vec_restore.vctrs_foobar = function(...) "dispatched-and-restored"
-  )
-
-  result <- vec_chop(foobar(NA))[[1]]
-  expect_equal(result, "dispatched-and-restored")
-})
-
-test_that("vec_chop() falls back to `[` for shaped objects with no proxy", {
-  x <- foobar(1)
-  dim(x) <- c(1, 1)
-  result <- vec_chop(x)[[1]]
-  expect_equal(result, x)
-})
-
-test_that("`indices` are validated", {
-  expect_error(vec_chop(1, 1), "`indices` must be a list of index values, or `NULL`")
-  expect_error(vec_chop(1, list(1.5)), class = "vctrs_error_cast_lossy")
-  expect_error(vec_chop(1, list(2)), class = "vctrs_error_subscript_oob")
-})
-
-test_that("size 0 `indices` list is allowed", {
-  expect_equal(vec_chop(1, list()), list())
-})
-
-test_that("individual index values of size 0 are allowed", {
-  expect_equal(vec_chop(1, list(integer())), list(numeric()))
-
-  df <- data.frame(a = 1, b = "1")
-  expect_equal(vec_chop(df, list(integer())), list(vec_ptype(df)))
-})
-
-test_that("data frame row names are kept when `indices` are used", {
-  x <- data_frame(x = 1:2, y = c("a", "b"))
-  rownames(x) <- c("r1", "r2")
-  result <- lapply(vec_chop(x, list(1, 1:2)), rownames)
-  expect_equal(result, list("r1", c("r1", "r2")))
-})
-
-test_that("vec_chop(<atomic>, indices =) can be equivalent to the default", {
-  x <- 1:5
-  indices <- as.list(vec_seq_along(x))
-  expect_equal(vec_chop(x, indices), vec_chop(x))
-})
-
-test_that("vec_chop(<data.frame>, indices =) can be equivalent to the default", {
-  x <- data.frame(x = 1:5)
-  indices <- as.list(vec_seq_along(x))
-  expect_equal(vec_chop(x, indices), vec_chop(x))
-})
-
-test_that("vec_chop(<array>, indices =) can be equivalent to the default", {
-  x <- array(1:8, c(2, 2, 2))
-  indices <- as.list(vec_seq_along(x))
-  expect_equal(vec_chop(x, indices), vec_chop(x))
-})
-
-test_that("`indices` can use names", {
-  x <- set_names(1:3, c("a", "b", "c"))
-
-  expect_equal(
-    vec_chop(x, list(1, 2:3)),
-    vec_chop(x, list("a", c("b", "c")))
-  )
-})
-
-test_that("`indices` can use array row names", {
-  x <- array(1:4, c(2, 2), dimnames = list(c("r1", "r2")))
-
-  expect_equal(
-    vec_chop(x, list("r1")),
-    vec_chop(x, list(1))
-  )
-})
-
-test_that("`indices` cannot use data frame row names", {
-  df <- data.frame(x = 1, row.names = "r1")
-  expect_error(vec_chop(df, list("r1")), "Can't use character")
-})
-
-test_that("fallback method with `indices` works", {
-  fctr <- factor(c("a", "b"))
-  indices <- list(1, c(1, 2))
-
-  expect_equal(
-    vec_chop(fctr, indices),
-    map(indices, vec_slice, x = fctr)
-  )
-})
-
-test_that("vec_chop() falls back to `[` for shaped objects with no proxy when indices are provided", {
-  x <- foobar(1)
-  dim(x) <- c(1, 1)
-  result <- vec_chop(x, list(1))[[1]]
-  expect_equal(result, x)
-})
-
 # vec_slice + compact_rep -------------------------------------------------
 
 # `i` is 1-based
@@ -648,10 +524,14 @@ test_that("names are repaired correctly with compact reps and `NA_integer_`", {
   expect_equal(vec_slice_rep(x, NA_integer_, 2L), expect)
 })
 
+test_that("names are recycled correctly with compact reps", {
+  expect_named(vec_slice_rep(c(x = 1L), 1L, 3L), c("x", "x", "x"))
+})
+
 test_that("vec_slice() with compact_reps work with Altrep classes", {
   skip_if(getRversion() < "3.5")
 
-  x <- .Call(vctrs_rle, c(foo = 10L, bar = 5L))
+  x <- .Call(vctrs_altrep_rle_Make, c(foo = 10L, bar = 5L))
 
   expect_equal(vec_slice_rep(x, 10L, 3L), rep("foo", 3))
 })
@@ -780,76 +660,71 @@ test_that("can subset S3 objects using the fallback method with compact seqs", {
 test_that("vec_slice() with compact_seqs work with Altrep classes", {
   skip_if(getRversion() < "3.5")
 
-  x <- .Call(vctrs_rle, c(foo = 2L, bar = 3L))
+  x <- .Call(vctrs_altrep_rle_Make, c(foo = 2L, bar = 3L))
 
   expect_equal(vec_slice_seq(x, 1L, 3L), c("foo", "bar", "bar"))
 })
 
-# vec_chop + compact_seq --------------------------------------------------
-
-# `start` is 0-based
-
-test_that("can chop base vectors with compact seqs", {
-  start <- 1L
-  size <- 2L
-  expect_identical(vec_chop_seq(lgl(1, 0, 1), start, size), list(lgl(0, 1)))
-  expect_identical(vec_chop_seq(int(1, 2, 3), start, size), list(int(2, 3)))
-  expect_identical(vec_chop_seq(dbl(1, 2, 3), start, size), list(dbl(2, 3)))
-  expect_identical(vec_chop_seq(cpl(1, 2, 3), start, size), list(cpl(2, 3)))
-  expect_identical(vec_chop_seq(chr("1", "2", "3"), start, size), list(chr("2", "3")))
-  expect_identical(vec_chop_seq(bytes(1, 2, 3), start, size), list(bytes(2, 3)))
-  expect_identical(vec_chop_seq(list(1, 2, 3), start, size), list(list(2, 3)))
+test_that("vec_slice() handles symbols and OO objects", {
+  expect_identical(vec_slice(c(a = 1, b = 2), quote(b)), c(b = 2))
+  expect_identical(vec_slice(c(a = 1, b = 2), factor("b")), c(b = 2))
+  expect_error(vec_slice(c(a = 1, b = 2), foobar("b")), class = "vctrs_error_subscript_type")
 })
 
-test_that("can chop with a decreasing compact seq", {
-  expect_equal(vec_chop_seq(int(1, 2, 3), 1L, 2L, FALSE), list(int(2, 1)))
-})
-
-test_that("can chop with multiple compact seqs", {
-  start <- c(1L, 0L)
-  size <- c(1L, 3L)
-
-  expect_equal(
-    vec_chop_seq(int(1, 2, 3), start, size),
-    list(int(2), int(1, 2, 3))
+test_that("vec_init() handles names in columns", {
+  expect_identical(
+    vec_init(data_frame(x = c(a = 1, b = 2)))$x,
+    named(na_dbl)
+  )
+  expect_identical(
+    vec_init(data_frame(x = c(1, 2)))$x,
+    na_dbl
   )
 })
 
-test_that("can chop S3 objects using the fallback method with compact seqs", {
-  x <- factor(c("a", "b", "c", "d"))
-  expect_equal(vec_chop_seq(x, 0L, 0L), list(vec_slice(x, integer())))
-  expect_equal(vec_chop_seq(x, 0L, 1L), list(vec_slice(x, 1L)))
-  expect_equal(vec_chop_seq(x, 2L, 2L), list(vec_slice(x, 3:4)))
+test_that("vec_slice() restores unrestored but named foreign classes", {
+  x <- foobar(c(x = 1))
+
+  expect_identical(vec_slice(x, 1), x)
+  expect_identical(vec_chop(x), list(x))
+  expect_identical(vec_chop(x, list(1)), list(x))
+  expect_identical(vec_ptype(x), foobar(named(dbl())))
+  expect_identical(vec_ptype(x), foobar(named(dbl())))
+  expect_identical(vec_ptype_common(x, x), foobar(named(dbl())))
+
+  out <- vec_ptype_common_fallback(x, x)
+  expect_true(is_common_class_fallback(out))
+  expect_identical(fallback_class(out), "vctrs_foobar")
 })
 
-test_that("vec_slice() works with Altrep classes with custom extract methods", {
-  skip_if(getRversion() < "3.5")
-
-  x <- .Call(vctrs_rle, c(foo = 10L, bar = 5L))
-
-  idx <- c(9, 10, 11)
-  expect_equal(vec_slice(x, idx), c("foo", "foo", "bar"))
+test_that("scalar type error is thrown when `vec_slice_impl()` is called directly (#1139)", {
+  x <- foobar(as.list(1:3))
+  expect_error(vec_slice_seq(x, 1L, 1L), class = "vctrs_error_scalar_type")
 })
 
-test_that("slice has informative error messages", {
-  verify_output(test_path("error", "test-slice.txt"), {
-    "# Unnamed vector with character subscript"
-    vec_slice(1:3, letters[1])
+test_that("column sizes are checked before slicing (#552)", {
+  x <- structure(list(a = 1, b = 2:3), row.names = 1:2, class = "data.frame")
+  expect_error(vctrs::vec_slice(x, 2), "must match the data frame size")
+})
 
-    "# Negative subscripts are checked"
-    vec_slice(1:3, -c(1L, NA))
-    vec_slice(1:3, c(-1L, 1L))
+test_that("base_vec_rep() slices data frames with the base::rep() UI", {
+  df <- data_frame(x = data_frame(y = 1:2))
+  expect_identical(
+    base_vec_rep(df, length.out = 4),
+    vec_slice(df, c(1:2, 1:2))
+  )
+})
 
-    "# oob error messages are properly constructed"
-    vec_slice(c(bar = 1), "foo")
-    "Multiple OOB indices"
-    vec_slice(letters, c(100, 1000))
-    vec_slice(letters, c(1, 100:103, 2, 104:110))
-    vec_slice(set_names(letters), c("foo", "bar"))
-    vec_slice(set_names(letters), toupper(letters))
+test_that("vec_size_assign() slices data frames with the base::rep() UI", {
+  df <- data_frame(x = data_frame(y = 1:3))
 
-    "# Can't index beyond the end of a vector"
-    vec_slice(1:2, 3L)
-    vec_slice(1:2, -3L)
-  })
+  expect_identical(
+    vec_size_assign(df, 2),
+    vec_slice(df, 1:2)
+  )
+
+  expect_identical(
+    vec_size_assign(df, 4),
+    vec_slice(df, c(1:3, NA))
+  )
 })
