@@ -2,6 +2,7 @@
 test_that("vec_as_location2() returns a position", {
   expect_identical(vec_as_location2(2, 2L), 2L)
   expect_identical(vec_as_location2("foo", 2L, c("bar", "foo")), 2L)
+  expect_identical(vec_as_location2("0", 4L, as.character(-1:2)), 2L)
 })
 
 test_that("vec_as_location2() requires integer or character inputs", {
@@ -11,10 +12,13 @@ test_that("vec_as_location2() requires integer or character inputs", {
     expect_error(vec_as_location2(env(), 10L), class = "vctrs_error_subscript_type")
     expect_error(vec_as_location2(foobar(), 10L), class = "vctrs_error_subscript_type")
     expect_error(vec_as_location2(2.5, 10L), class = "vctrs_error_subscript_type")
+    expect_error(vec_as_location2(Inf, 10L), class = "vctrs_error_subscript_type")
+    expect_error(vec_as_location2(-Inf, 10L), class = "vctrs_error_subscript_type")
 
     "Idem with custom `arg`"
     expect_error(vec_as_location2(foobar(), 10L, arg = "foo"), class = "vctrs_error_subscript_type")
     expect_error(vec_as_location2(2.5, 3L, arg = "foo"), class = "vctrs_error_subscript_type")
+    expect_error(with_tibble_rows(vec_as_location2(TRUE)), class = "vctrs_error_subscript_type")
   })
 })
 
@@ -26,6 +30,7 @@ test_that("vec_as_location() requires integer, character, or logical inputs", {
     expect_error(vec_as_location(2.5, 10L), class = "vctrs_error_subscript_type")
     expect_error(vec_as_location(list(), 10L), class = "vctrs_error_subscript_type")
     expect_error(vec_as_location(function() NULL, 10L), class = "vctrs_error_subscript_type")
+    expect_error(vec_as_location(Sys.Date(), 3L), class = "vctrs_error_subscript_type")
 
     "Idem with custom `arg`"
     expect_error(vec_as_location(env(), 10L, arg = "foo"), class = "vctrs_error_subscript_type")
@@ -42,8 +47,7 @@ test_that("vec_as_location2() and vec_as_location() require integer- or characte
 
   # Define subtype of logical and integer
   local_methods(
-    vec_ptype2.vctrs_foobar = function(x, y, ...) UseMethod("vec_ptype2.vctrs_foobar", y),
-    vec_ptype2.vctrs_foobar.default = function(x, y, ...) vec_default_ptype2(x, y, ...),
+    vec_ptype2.vctrs_foobar = function(x, y, ...) UseMethod("vec_ptype2.vctrs_foobar"),
     vec_ptype2.vctrs_foobar.logical = function(x, y, ...) logical(),
     vec_ptype2.vctrs_foobar.integer = function(x, y, ...) integer(),
     vec_ptype2.logical.vctrs_foobar = function(x, y, ...) logical(),
@@ -166,6 +170,23 @@ test_that("num_as_location() optionally forbids negative indices", {
   expect_error(num_as_location(c(1, -10), 2L, negative = "error"), class = "vctrs_error_subscript_type")
 })
 
+test_that("num_as_location() optionally ignores zero indices", {
+  expect_identical(num_as_location(c(1, 0), 2L, zero = "ignore"), c(1L, 0L))
+})
+
+test_that("num_as_location() optionally forbids zero indices", {
+  verify_errors({
+    expect_error(
+      num_as_location(0L, 1L, zero = "error"),
+      class = "vctrs_error_subscript_type"
+    )
+    expect_error(
+      num_as_location(c(0, 0, 0, 0, 0, 0), 1, zero = "error"),
+      class = "vctrs_error_subscript_type"
+    )
+  })
+})
+
 test_that("vec_as_location() handles NULL", {
   expect_identical(
     vec_as_location(NULL, 10),
@@ -216,11 +237,15 @@ test_that("character subscripts require named vectors", {
 
 test_that("can optionally extend beyond the end", {
   expect_error(num_as_location(1:5, 3), class = "vctrs_error_subscript_oob")
+
   expect_identical(num_as_location(1:5, 3, oob = "extend"), 1:5)
-  expect_identical(num_as_location(c(1:5, 7, 6), 3, oob = "extend"), c(1:5, 7L, 6L))
-  expect_identical(num_as_location(c(1, NA, 3), 2, oob = "extend"), c(1L, NA, 3L))
+  expect_identical(num_as_location(4:5, 3, oob = "extend"), 4:5)
 
   verify_errors({
+    expect_error(
+      num_as_location(3, 1, oob = "extend"),
+      class = "vctrs_error_subscript_oob"
+    )
     expect_error(
       num_as_location(c(1, 3), 1, oob = "extend"),
       class = "vctrs_error_subscript_oob"
@@ -240,6 +265,12 @@ test_that("can optionally extend beyond the end", {
   })
 })
 
+test_that("can extend beyond the end consecutively but non-monotonically (#1166)", {
+  expect_identical(num_as_location(6:4, 3, oob = "extend"), 6:4)
+  expect_identical(num_as_location(c(1:5, 7, 6), 3, oob = "extend"), c(1:5, 7L, 6L))
+  expect_identical(num_as_location(c(1, NA, 4, 3), 2, oob = "extend"), c(1L, NA, 4L, 3L))
+})
+
 test_that("missing values are supported in error formatters", {
   verify_errors({
     expect_error(
@@ -309,6 +340,10 @@ test_that("can customise subscript type errors", {
       num_as_location(c(1, 4), 2, oob = "extend", arg = "foo"),
       class = "vctrs_error_subscript_oob"
     )
+    expect_error(
+      num_as_location(0, 1, zero = "error", arg = "foo"),
+      class = "vctrs_error_subscript_type"
+    )
 
     "With tibble columns"
     expect_error(
@@ -347,6 +382,10 @@ test_that("can customise subscript type errors", {
       with_tibble_cols(num_as_location(c(1, 4), 2, oob = "extend")),
       class = "vctrs_error_subscript_oob"
     )
+    expect_error(
+      with_tibble_cols(num_as_location(0, 1, zero = "error")),
+      class = "vctrs_error_subscript_type"
+    )
   })
 })
 
@@ -397,8 +436,27 @@ test_that("can customise OOB errors", {
   })
 })
 
+test_that("num_as_location() requires non-S3 inputs", {
+  expect_error(num_as_location(factor("foo"), 2), "must be a numeric vector")
+})
+
+test_that("vec_as_location() checks dimensionality", {
+  verify_errors({
+    expect_error(vec_as_location(matrix(TRUE, nrow = 1), 3L), class = "vctrs_error_subscript_type")
+    expect_error(vec_as_location(array(TRUE, dim = c(1, 1, 1)), 3L), class = "vctrs_error_subscript_type")
+    expect_error(with_tibble_rows(vec_as_location(matrix(TRUE, nrow = 1), 3L)), class = "vctrs_error_subscript_type")
+  })
+})
+
+test_that("vec_as_location() works with vectors of dimensionality 1", {
+  expect_identical(vec_as_location(array(TRUE, dim = 1), 3L), 1:3)
+})
+
 test_that("conversion to locations has informative error messages", {
   verify_output(test_path("error", "test-subscript-loc.txt"), {
+    "# vec_as_location() UI"
+    vec_as_location(1, 1L, missing = "bogus")
+
     "# vec_as_location() checks for mix of negative and missing locations"
     vec_as_location(-c(1L, NA), 30)
     vec_as_location(-c(1L, rep(NA, 10)), 30)
@@ -407,9 +465,19 @@ test_that("conversion to locations has informative error messages", {
     vec_as_location(c(-1L, 1L), 30)
     vec_as_location(c(-1L, rep(1L, 10)), 30)
 
+    "# num_as_location() UI"
+    num_as_location(1, 1L, missing = "bogus")
+    num_as_location(1, 1L, negative = "bogus")
+    num_as_location(1, 1L, oob = "bogus")
+    num_as_location(1, 1L, zero = "bogus")
+
     "# num_as_location() optionally forbids negative indices"
     num_as_location(dbl(1, -1), 2L, negative = "error")
 
+    "# num_as_location() optionally forbids zero indices"
+    num_as_location(0L, 1L, zero = "error")
+    num_as_location(c(0, 0, 0, 0, 0, 0), 1, zero = "error")
+
     "# logical subscripts must match size of indexed vector"
     vec_as_location(c(TRUE, FALSE), 3)
 
@@ -423,20 +491,27 @@ test_that("conversion to locations has informative error messages", {
     vec_as_location(2.5, 3L)
     vec_as_location(list(), 10L)
     vec_as_location(function() NULL, 10L)
+    vec_as_location(Sys.Date(), 3L)
     "Idem with custom `arg`"
     vec_as_location(env(), 10L, arg = "foo")
     vec_as_location(foobar(), 10L, arg = "foo")
     vec_as_location(2.5, 3L, arg = "foo")
 
+    "# vec_as_location2() UI"
+    vec_as_location2(1, 1L, missing = "bogus")
+
     "# vec_as_location2() requires integer or character inputs"
     vec_as_location2(TRUE, 10L)
     vec_as_location2(mtcars, 10L)
     vec_as_location2(env(), 10L)
     vec_as_location2(foobar(), 10L)
     vec_as_location2(2.5, 3L)
+    vec_as_location2(Inf, 10L)
+    vec_as_location2(-Inf, 10L)
     "Idem with custom `arg`"
     vec_as_location2(foobar(), 10L, arg = "foo")
     vec_as_location2(2.5, 3L, arg = "foo")
+    with_tibble_rows(vec_as_location2(TRUE))
 
     "# vec_as_location2() requires length 1 inputs"
     vec_as_location2(1:2, 2L)
@@ -467,6 +542,10 @@ test_that("conversion to locations has informative error messages", {
     vec_as_location("foo", 1L, names = "bar")
     vec_as_location2("foo", 1L, names = "bar")
 
+    "# num_as_location() UI"
+    num_as_location(1, 1L, missing = "bogus")
+    num_as_location(1, 1L, negative = "bogus")
+
     "# can optionally extend beyond the end"
     num_as_location(c(1, 3), 1, oob = "extend")
     num_as_location(c(1:5, 7), 3, oob = "extend")
@@ -488,6 +567,7 @@ test_that("conversion to locations has informative error messages", {
     vec_as_location(c(-1, NA), 3, arg = "foo")
     vec_as_location(c(-1, 1), 3, arg = "foo")
     num_as_location(c(1, 4), 2, oob = "extend", arg = "foo")
+    num_as_location(0, 1, zero = "error", arg = "foo")
     "With tibble columns"
     with_tibble_cols(num_as_location(-1, 2, negative = "error"))
     with_tibble_cols(num_as_location2(-1, 2, negative = "error"))
@@ -498,6 +578,7 @@ test_that("conversion to locations has informative error messages", {
     with_tibble_cols(vec_as_location(c(-1, NA), 3))
     with_tibble_cols(vec_as_location(c(-1, 1), 3))
     with_tibble_cols(num_as_location(c(1, 4), 2, oob = "extend"))
+    with_tibble_cols(num_as_location(0, 1, zero = "error"))
 
     "# can customise OOB errors"
     vec_slice(set_names(letters), "foo")
@@ -517,5 +598,10 @@ test_that("conversion to locations has informative error messages", {
     vec_as_location(c(1, NA), 2, missing = "error")
     vec_as_location(c(1, NA, 2, NA), 2, missing = "error", arg = "foo")
     with_tibble_cols(vec_as_location(c(1, NA, 2, NA), 2, missing = "error"))
+
+    "# vec_as_location() checks dimensionality"
+    vec_as_location(matrix(TRUE, nrow = 1), 3L)
+    vec_as_location(array(TRUE, dim = c(1, 1, 1)), 3L)
+    with_tibble_rows(vec_as_location(matrix(TRUE, nrow = 1), 3L))
   })
 })
