@@ -1,4 +1,3 @@
-context("vctrs")
 
 test_that("generics are extensible", {
   expect_error(vec_cast(NA, NA, NA), class = "rlib_error_dots_nonempty")
@@ -11,4 +10,5 @@ test_that("generics are extensible", {
   expect_error(vec_ptype_full(NA, NA), class = "rlib_error_dots_nonempty")
   expect_error(vec_arith(NA, NA, NA, NA), class = "rlib_error_dots_nonempty")
   expect_error(vec_ptype_finalise(NA, NA), class = "rlib_error_dots_nonempty")
+  expect_error(vec_assign(NA, NA, NA, NA), class = "rlib_error_dots_nonempty")
 })
